@@ -82,7 +82,6 @@ use Errno qw(ENOENT EACCES);
 use File::Basename;
 use File::Path;
 use File::Spec 0.8;
-use File::Copy;
 use Time::HiRes qw(time);
 use Cwd;
 use Config;
@@ -94,8 +93,8 @@ use vars qw{
   @site_rules_path
 };
 
-$VERSION = "3.003001";      # update after release (same format as perl $])
-# $IS_DEVEL_BUILD = 1;        # change for release versions
+$VERSION = "3.003002";      # update after release (same format as perl $])
+#$IS_DEVEL_BUILD = 1;        # change for release versions
 
 # Used during the prerelease/release-candidate part of the official release
 # process. If you hacked up your SA, you should add a version_tag to your .cf
@@ -105,11 +104,11 @@ $VERSION = "3.003001";      # update after release (same format as perl $])
 @ISA = qw();
 
 # SUB_VERSION is now just <yyyy>-<mm>-<dd>
-$SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2010-03-16 13:15:21 +0000 (Tue, 16 Mar 2010) $ updated by SVN'))[1];
+$SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2011-06-06 23:57:17 +0000 (Mon, 06 Jun 2011) $ updated by SVN'))[1];
 
 if (defined $IS_DEVEL_BUILD && $IS_DEVEL_BUILD) {
   push(@EXTRA_VERSION,
-       ('r' . qw{$LastChangedRevision: 923725 $ updated by SVN}[1]));
+       ('r' . qw{$LastChangedRevision: 1132835 $ updated by SVN}[1]));
 }
 
 sub Version {
@@ -490,10 +489,16 @@ sub parse {
   my $timer = $self->time_method("parse");
 
   my $master_deadline;
+  # passed in at a function call
   if (ref $suppl_attrib && exists $suppl_attrib->{master_deadline}) {
     $master_deadline = $suppl_attrib->{master_deadline};  # may be undef
-  } elsif ($self->{conf}->{time_limit}) {  # defined and nonzero
-    $master_deadline = $start_time + $self->{conf}->{time_limit};
+  }
+  # found in a config file - overrides passed-in number if lower
+  if ($self->{conf}->{time_limit}) {  # defined and nonzero
+    my $time_limit_deadline = $start_time + $self->{conf}->{time_limit};
+    if (!defined $master_deadline || $time_limit_deadline < $master_deadline) {
+      $master_deadline = $time_limit_deadline;
+    }
   }
   if (defined $master_deadline) {
     dbg("config: time limit %.1f s", $master_deadline - $start_time);
